@@ -39,9 +39,11 @@ use iso_fortran_env, only: int64
 use fpm_compiler, only: compiler_t, archiver_t, debug
 use fpm_dependency, only: dependency_tree_t
 use fpm_strings, only: string_t, str, len_trim, upper, operator(==)
-use fpm_toml, only: serializable_t, toml_table, toml_stat, set_value, set_list, get_value, &
+use tomlf, only: toml_table, toml_stat
+use fpm_toml, only: serializable_t, set_value, set_list, get_value, &
                     & get_list, add_table, toml_key, add_array, set_string
 use fpm_error, only: error_t, fatal_error
+use fpm_environment, only: OS_WINDOWS,OS_MACOS
 use fpm_manifest_preprocess, only: preprocess_config_t
 implicit none
 
@@ -171,7 +173,10 @@ type, extends(serializable_t) :: package_t
     type(fortran_features_t) :: features
 
     contains
-
+    
+        !> Check if a package will create a library
+        procedure :: has_library => package_has_library
+    
         !> Serialization interface
         procedure :: serializable_is_same => package_is_same
         procedure :: dump_to_toml   => package_dump_to_toml
@@ -233,7 +238,10 @@ type, extends(serializable_t) :: fpm_model_t
     type(string_t) :: module_prefix
 
     contains
-
+    
+        !> Get target link flags
+        procedure :: get_package_libraries_link
+    
         !> Serialization interface
         procedure :: serializable_is_same => model_is_same
         procedure :: dump_to_toml   => model_dump_to_toml
@@ -243,7 +251,6 @@ end type fpm_model_t
 
 contains
 
-
 function info_package(p) result(s)
     ! Returns representation of package_t
     type(package_t), intent(in) :: p
@@ -480,16 +487,36 @@ logical function srcfile_is_same(this,that)
 
     select type (other=>that)
        type is (srcfile_t)
-
-          if (.not.(this%file_name==other%file_name)) return
-          if (.not.(this%exe_name==other%exe_name)) return
+          if (allocated(this%file_name).neqv.allocated(other%file_name)) return
+          if (allocated(this%file_name)) then
+            if (.not.(this%file_name==other%file_name)) return
+          end if
+          if (allocated(this%exe_name).neqv.allocated(other%exe_name)) return
+          if (allocated(this%exe_name)) then
+            if (.not.(this%exe_name==other%exe_name)) return
+          end if
           if (.not.(this%unit_scope==other%unit_scope)) return
-          if (.not.(this%modules_provided==other%modules_provided)) return
+          if (allocated(this%modules_provided).neqv.allocated(other%modules_provided)) return
+          if (allocated(this%modules_provided)) then
+            if (.not.(this%modules_provided==other%modules_provided)) return
+          end if
           if (.not.(this%unit_type==other%unit_type)) return
-          if (.not.(this%parent_modules==other%parent_modules)) return
-          if (.not.(this%modules_used==other%modules_used)) return
-          if (.not.(this%include_dependencies==other%include_dependencies)) return
-          if (.not.(this%link_libraries==other%link_libraries)) return
+          if (allocated(this%parent_modules).neqv.allocated(other%parent_modules)) return
+          if (allocated(this%parent_modules)) then
+            if (.not.(this%parent_modules==other%parent_modules)) return
+          end if
+          if (allocated(this%modules_used).neqv.allocated(other%modules_used)) return
+          if (allocated(this%modules_used)) then
+            if (.not.(this%modules_used==other%modules_used)) return
+          end if
+          if (allocated(this%include_dependencies).neqv.allocated(other%include_dependencies)) return
+          if (allocated(this%include_dependencies)) then
+            if (.not.(this%include_dependencies==other%include_dependencies)) return
+          end if
+          if (allocated(this%link_libraries).neqv.allocated(other%link_libraries)) return
+          if (allocated(this%link_libraries)) then
+            if (.not.(this%link_libraries==other%link_libraries)) return
+          end if
           if (.not.(this%digest==other%digest)) return
 
        class default
@@ -598,7 +625,10 @@ logical function fft_is_same(this,that)
 
            if (.not.(this%implicit_typing.eqv.other%implicit_typing)) return
            if (.not.(this%implicit_external.eqv.other%implicit_external)) return
-           if (.not.(this%source_form==other%source_form)) return
+           if (allocated(this%source_form).neqv.allocated(other%source_form)) return
+           if (allocated(this%source_form)) then
+             if (.not.(this%source_form==other%source_form)) return
+           end if
 
        class default
           ! Not the same type
@@ -665,9 +695,11 @@ logical function package_is_same(this,that)
 
     select type (other=>that)
        type is (package_t)
-
-           if (.not.(this%name==other%name)) return
-           if (.not.(allocated(this%sources).eqv.allocated(other%sources))) return
+           if (allocated(this%name).neqv.allocated(other%name)) return
+           if (allocated(this%name)) then
+              if (.not.(this%name==other%name)) return
+           end if
+           if (allocated(this%sources).neqv.allocated(other%sources)) return
            if (allocated(this%sources)) then
               if (.not.(size(this%sources)==size(other%sources))) return
               do ii = 1, size(this%sources)
@@ -676,7 +708,10 @@ logical function package_is_same(this,that)
            end if
 
            if (.not.(this%preprocess==other%preprocess)) return
-           if (.not.(this%version==other%version)) return
+           if (allocated(this%version).neqv.allocated(other%version)) return
+           if (allocated(this%version)) then
+             if (.not.(this%version==other%version)) return
+           end if
 
            !> Module naming
            if (.not.(this%enforce_module_names.eqv.other%enforce_module_names)) return
@@ -850,7 +885,10 @@ logical function model_is_same(this,that)
     select type (other=>that)
        type is (fpm_model_t)
 
-           if (.not.(this%package_name==other%package_name)) return
+           if ((allocated(this%package_name).neqv.allocated(other%package_name))) return
+           if (allocated(this%package_name)) then
+             if (.not.(this%package_name==other%package_name)) return
+           end if
            if (.not.(allocated(this%packages).eqv.allocated(other%packages))) return
            if (allocated(this%packages)) then
                if (.not.(size(this%packages)==size(other%packages))) return
@@ -861,14 +899,38 @@ logical function model_is_same(this,that)
 
            if (.not.(this%compiler==other%compiler)) return
            if (.not.(this%archiver==other%archiver)) return
-           if (.not.(this%fortran_compile_flags==other%fortran_compile_flags)) return
-           if (.not.(this%c_compile_flags==other%c_compile_flags)) return
-           if (.not.(this%cxx_compile_flags==other%cxx_compile_flags)) return
-           if (.not.(this%link_flags==other%link_flags)) return
-           if (.not.(this%build_prefix==other%build_prefix)) return
-           if (.not.(this%include_dirs==other%include_dirs)) return
-           if (.not.(this%link_libraries==other%link_libraries)) return
-           if (.not.(this%external_modules==other%external_modules)) return
+           if (allocated(this%fortran_compile_flags).neqv.allocated(other%fortran_compile_flags)) return
+           if (allocated(this%fortran_compile_flags)) then
+             if (.not.(this%fortran_compile_flags==other%fortran_compile_flags)) return
+           end if
+           if (allocated(this%c_compile_flags).neqv.allocated(other%c_compile_flags)) return
+           if (allocated(this%c_compile_flags)) then
+             if (.not.(this%c_compile_flags==other%c_compile_flags)) return
+           end if
+           if (allocated(this%cxx_compile_flags).neqv.allocated(other%cxx_compile_flags)) return
+           if (allocated(this%cxx_compile_flags)) then
+             if (.not.(this%cxx_compile_flags==other%cxx_compile_flags)) return
+           end if
+           if (allocated(this%link_flags).neqv.allocated(other%link_flags)) return
+           if (allocated(this%link_flags)) then
+             if (.not.(this%link_flags==other%link_flags)) return
+           end if
+           if (allocated(this%build_prefix).neqv.allocated(other%build_prefix)) return
+           if (allocated(this%build_prefix)) then
+             if (.not.(this%build_prefix==other%build_prefix)) return
+           end if
+           if (allocated(this%include_dirs).neqv.allocated(other%include_dirs)) return
+           if (allocated(this%include_dirs)) then
+             if (.not.(this%include_dirs==other%include_dirs)) return
+           end if
+           if (allocated(this%link_libraries).neqv.allocated(other%link_libraries)) return
+           if (allocated(this%link_libraries)) then
+             if (.not.(this%link_libraries==other%link_libraries)) return
+           end if
+           if (allocated(this%external_modules).neqv.allocated(other%external_modules)) return
+           if (allocated(this%external_modules)) then
+             if (.not.(this%external_modules==other%external_modules)) return
+           end if
            if (.not.(this%deps==other%deps)) return
            if (.not.(this%include_tests.eqv.other%include_tests)) return
            if (.not.(this%enforce_module_names.eqv.other%enforce_module_names)) return
@@ -1080,4 +1142,80 @@ subroutine model_load_from_toml(self, table, error)
 
 end subroutine model_load_from_toml
 
+function get_package_libraries_link(model, package_name, prefix, exclude_self, dep_IDs, error) result(r)
+    class(fpm_model_t), intent(in) :: model
+    character(*), intent(in) :: package_name
+    type(error_t), allocatable, intent(out) :: error
+    character(*), intent(in) :: prefix
+    !> Option to exclude linking to the given package (needed building it as a library)
+    logical, optional, intent(in) :: exclude_self
+    !> Optionally export the list of dependency IDs
+    integer, allocatable, optional, intent(out) :: dep_IDs(:)
+    character(len=:), allocatable :: r
+    
+    integer :: id,ndep,i
+    logical :: no_root
+    integer, allocatable :: sorted_package_IDs(:)
+    logical, allocatable :: has_lib(:)
+    type(string_t), allocatable :: package_deps(:)
+    
+    ! Get dependency ID of this target 
+    id = model%deps%find(package_name)
+    if (id<=0) then 
+        call fatal_error(error, "Internal error: shared library "//package_name// &
+                                " does not correspond to a package")
+        return
+    end if
+    
+    ! Get ordered IDs of the shared libraries that should be linked against
+    call model%deps%local_link_order(id, sorted_package_IDs, error)
+    if (allocated(error)) return
+    
+    ! Get names of the package dependencies
+    ndep = size(sorted_package_IDs)
+    
+    if (ndep<=0) then 
+       r = prefix
+       if (present(dep_IDs)) allocate(dep_IDs(0))
+       return 
+    end if
+    
+    ! Optional exclusion of self (top-level) package
+    no_root = .false.
+    if (present(exclude_self)) no_root = exclude_self
+    if (no_root) then 
+        sorted_package_IDs = pack(sorted_package_IDs, sorted_package_IDs /= id)
+        ndep = size(sorted_package_IDs)
+    endif
+    
+    ! Exclusion of package IDs marked "empty" (i.e. they contain no sources)
+    has_lib = model%packages%has_library()
+    
+    if (any(.not.has_lib)) then 
+        sorted_package_IDs = pack(sorted_package_IDs, has_lib(sorted_package_IDs))
+        ndep = size(sorted_package_IDs)
+    end if
+    
+    package_deps = [(string_t(model%deps%dep(sorted_package_IDs(i))%name),i=1,ndep)]
+    
+    r = model%compiler%enumerate_libraries(prefix, package_deps)
+    
+    ! If requested, export the list of dependency IDs
+    if (present(dep_IDs)) call move_alloc(from=sorted_package_IDs,to=dep_IDs)
+    
+end function get_package_libraries_link
+
+!> Check whether a package has an object library
+elemental logical function package_has_library(self) result(has_library)
+    class(package_t), intent(in) :: self
+    
+    if (allocated(self%sources)) then 
+        has_library = any(self%sources%unit_scope==FPM_SCOPE_LIB)
+    else
+        has_library = .false.
+    end if
+    
+end function package_has_library
+
+
 end module fpm_model
